///////////////////////////////////////////////////////////////
// The PC Benchmark (pcb)
// (c) 2015, Marko Oette (www.oette.info)
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef __TESTS_H_INCLUDED__
#define __TESTS_H_INCLUDED__

// preprocessor helper functions
#define STRINGIFY(x) #x
#define TOSTRING(x) STRINGIFY(x)

// Code variant
#ifdef _DEBUG
	#define __OPTIMIZE__ "DEBUG" 	
#else
	#define __OPTIMIZE__ "RELEASE"
#endif

// Target system dependent settings
#ifdef _WIN32
	#define _CRT_SECURE_NO_WARNINGS 1	
	#include "windows.h"

	#define __VERSION__ MSVC _MSC_VER

	#if (_MSC_VER > 900)		// Everything newer than VC++ 2.0
		// iostream.h 
		#include <iostream>
		#include <strstream>
		#include <string>
		#include <fstream>
		using namespace std;
		#define _USE_W32PERFCOUNTER 1
	#else
		// Older MSVC Variants (1.5 / 2) do not know Boolean
		#define bool short
		#define true 1
		#define false 0

		// includes that were renamed after vc 2
		#include <iostream.h>
		#include <strstrea.h>
		#include <string.h>
		#include <fstream.h>
		#include <stdio.h>
	#endif
	
	#ifdef _M_X64	// Use 64 Bit Datatypes
		#define BITNESS 			64
		#define INT_TYPE			__int64		
		#define FP_TYPE				double	
		#define LNG_TYPE			long long
		#define MEMCPY				memcpy
		#define MEMSET				memset
	#else // assume 32 bit
		#ifdef SSE32	
			#define BITNESS 			32
			#define INT_TYPE			__int64		
			#define FP_TYPE				double	
			#define LNG_TYPE			size_t
			#define MEMCPY				memcpy
			#define MEMSET				memset
		#else
			#define BITNESS 			32
			#define INT_TYPE			long		
			#define FP_TYPE				double	
			#define LNG_TYPE			size_t
			#define MEMCPY				memcpy
			#define MEMSET				memset
		#endif
	#endif

	
	#define MAX_SIZE			(size_t)-1		// This is the maximum allocatable block size per plattform

#elif _DOS
	#define __VERSION__ MSVC _MSC_VER          

	#define BITNESS 			16
	#define INT_TYPE			long
	#define FP_TYPE				double	     
	
	#define MEMCPY				memcpy
	#define MEMSET				memset
#endif

#define PROC_PRIO_NORMAL 1
#define PROC_PRIO_HIGH   2
#define PROC_PRIO_REAL   3

// Forward defines
LNG_TYPE timedTest( int(*pfn)( LNG_TYPE, LNG_TYPE, const void* ), LNG_TYPE, LNG_TYPE, const void* );
LNG_TYPE timedMTTest(unsigned int threads, int(*pfn)(LNG_TYPE, LNG_TYPE, const void*), LNG_TYPE p1, LNG_TYPE p2, const void* buf);
bool setProcessPriority( int );
void printSysInfo(ofstream &);

#endif // #ifndef __TESTS_H_INCLUDED__
